import uuid

from django.db import models

from django.utils.translation import ugettext_lazy as _

class PasswordRecoveryToken(models.Model):
  user = models.ForeignKey('User', blank=True, null=True)
  token = models.CharField(_('Token'), max_length=128, null=False, blank=False)
  created_date = models.DateTimeField(_('Created date'), auto_now_add=True, blank=True, null=True)
  used_date = models.DateTimeField(_('Used date'), default=None, blank=True, null=True)

  def save(self, *args, **kwargs):
    if not self.pk:
      self.token = uuid.uuid4()
      self.user.mailing().sendRecoveryToken({'token': self})

    super(PasswordRecoveryToken, self).save(*args, **kwargs)

  class Meta:
    app_label = 'ovp_users'



from ovp_users import emails
from ovp_users.models.profile import get_profile_model

from django.contrib.auth.models import AbstractBaseUser
from django.contrib.auth.models import BaseUserManager
from django.contrib.auth.models import PermissionsMixin

from django.db import models
from django.utils import timezone
from django.utils.text import slugify

from django.utils.translation import ugettext_lazy as _

import uuid
from shortuuid.main import encode as encode_uuid

from random import randint


class UserManager(BaseUserManager):
  def create_user(self, email, password=None, **extra_fields):
    now = timezone.now()
    if not email:
      raise ValueError('The given email address must be set.')
    email = UserManager.normalize_email(email)
    user = self.model(email=email, is_staff=False,
                      is_active=True, last_login=now,
                      joined_date=now, **extra_fields)

    user.password = password
    user.save()
    return user

  def create_superuser(self, email, password, **extra_fields):
    user = self.create_user(email, password, **extra_fields)
    user.is_staff = True
    user.is_active = True
    user.is_superuser = True
    user.save()
    return user

  class Meta:
    app_label = 'ovp_user'

class User(AbstractBaseUser, PermissionsMixin):
  uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)
  email = models.EmailField(_('Email'), max_length=190, unique=True)
  name = models.CharField(_('Name'), max_length=200, null=False, blank=False)
  slug = models.SlugField(_('Slug'), max_length=100, null=True, blank=True, unique=True)
  phone = models.CharField(_('Phone'), max_length=30, null=True, blank=True)
  avatar = models.ForeignKey('ovp_uploads.UploadedImage', blank=False, null=True, related_name='avatar_user', verbose_name=_('avatar'))
  public = models.BooleanField(_('Public'), default=True)

  locale = models.CharField(_('Locale'), max_length=8, null=False, blank=True, default='en')

  is_staff = models.BooleanField(_('Staff'), default=False)
  is_superuser = models.BooleanField(_('Superuser'), default=False)
  is_active = models.BooleanField(_('Active'), default=True)
  is_email_verified = models.BooleanField(_('Email verified'), default=False)

  joined_date = models.DateTimeField(_('Joined date'), auto_now_add=True, null=True, blank=True)
  modified_date = models.DateTimeField(_('Modified date'), auto_now=True, null=True, blank=True)

  objects = UserManager()
  USERNAME_FIELD = 'email'

  class Meta:
    app_label = 'ovp_users'

  def __init__(self, *args, **kwargs):
    super(User, self).__init__(*args, **kwargs)
    self.__original_password = self.password

  def mailing(self, async_mail=None):
    return emails.UserMail(self, async_mail)

  def save(self, *args, **kwargs):
    hash_password = False

    if not self.pk:
      self.slug = encode_uuid(self.uuid)
      hash_password = True
      self.mailing().sendWelcome()
    else:
      # checks if password has changed and if it was set by set_password
      if self.__original_password != self.password and not self.check_password(self._password):
        hash_password = True

    if hash_password:
      self.set_password(self.password) # hash it
      self.__original_password = self.password

    super(User, self).save(*args, **kwargs)

  def get_full_name(self):
    return self.name

  def get_short_name(self):
    return self.name

  @property
  def profile(self):
    model = get_profile_model()
    related_field_name = model._meta.get_field('user').related_query_name()
    try:
      return getattr(self, related_field_name, None)
    except model.DoesNotExist:
      return None



from ovp_users.models.profile import get_profile_model
from ovp_users.models.profile import gender_choices
from ovp_users.helpers import get_settings, import_from_string

from ovp_core.models import Skill
from ovp_core.models import Cause
from ovp_core.serializers.skill import SkillSerializer, SkillAssociationSerializer
from ovp_core.serializers.cause import CauseSerializer, CauseAssociationSerializer

from rest_framework import serializers


class ProfileCreateUpdateSerializer(serializers.ModelSerializer):
  # we do not allow the user to create skills, only associate with
  # existing skills, so we do it manually on .create method
  skills = SkillAssociationSerializer(many=True, required=False)
  causes = CauseAssociationSerializer(many=True, required=False)
  gender = serializers.ChoiceField(choices=gender_choices)

  class Meta:
    model = get_profile_model()
    fields = ['full_name', 'about', 'skills', 'causes', 'gender']

  def create(self, validated_data):
    skills = validated_data.pop('skills', [])
    causes = validated_data.pop('causes', [])

    # Create profile
    profile = get_profile_model().objects.create(**validated_data)

    # Associate skills
    for skill in skills:
      s = Skill.objects.get(pk=skill['id'])
      profile.skills.add(s)

    # Associate causes
    for cause in causes:
      c = Cause.objects.get(pk=cause['id'])
      profile.causes.add(c)

    return profile


  def update(self, instance, validated_data):
    skills = validated_data.pop('skills', None)
    causes = validated_data.pop('causes', None)

    # Associate skills
    if skills:
      instance.skills.clear()
      for skill in skills:
        s = Skill.objects.get(pk=skill['id'])
        instance.skills.add(s)

    # Associate causes
    if causes:
      instance.causes.clear()
      for cause in causes:
        c = Cause.objects.get(pk=cause['id'])
        instance.causes.add(c)

    return super(ProfileCreateUpdateSerializer, self).update(instance, validated_data)

class ProfileRetrieveSerializer(serializers.ModelSerializer):
  skills = SkillSerializer(many=True)
  causes = CauseSerializer(many=True)
  gender = serializers.ChoiceField(choices=gender_choices)

  class Meta:
    model = get_profile_model()
    fields = ['full_name', 'about', 'skills', 'causes', 'gender']

class ProfileSearchSerializer(serializers.ModelSerializer):
  skills = SkillSerializer(many=True)
  causes = CauseSerializer(many=True)
  gender = serializers.ChoiceField(choices=gender_choices)

  class Meta:
    model = get_profile_model()
    fields = ['full_name', 'skills', 'causes', 'gender']


def get_profile_serializers():
  s = get_settings()
  serializers = s.get('PROFILE_SERIALIZER_TUPLE', None)
  if isinstance(serializers, tuple):
    return [import_from_string(s) for s in serializers]
  return (ProfileCreateUpdateSerializer, ProfileRetrieveSerializer, ProfileSearchSerializer)




from django.conf import settings
import importlib

def get_settings(string="OVP_USERS"):
  return getattr(settings, string, {})

def import_from_string(val):
  try:
    # Nod to tastypie's use of importlib.
    parts = val.split('.')
    module_path, class_name = '.'.join(parts[:-1]), parts[-1]
    module = importlib.import_module(module_path)
    return getattr(module, class_name)
  except ImportError as e:
    msg = "Could not import '%s' for setting. %s: %s." % (val, e.__class__.__name__, e)
    raise ImportError(msg)



from django.test import TestCase

from ovp_users.models import User

from shortuuid.main import encode as encode_uuid

class TestUserManager(TestCase):
  def test_create_user_without_email(self):
    """ Assert that UserManager doesn't create users without email"""
    with self.assertRaises(ValueError) as context:
      User.objects.create_user(None, 'validpassword')

    self.assertTrue('The given email address must be set.' == str(context.exception))
    #self.assertTrue(user.id > 0)

  def test_create_user(self):
    """ Assert that UserManager can create user"""
    user = User.objects.create_user('test_create_user@test.com', 'validpassword')
    self.assertTrue(user.id > 0)

  def test_create_superuser(self):
    """ Assert that UserManager can create super user"""
    user = User.objects.create_superuser('test_create_superuser@test.com', 'validpassword')
    self.assertTrue(user.id > 0)

class TestUserModel(TestCase):
  def test_short_name(self):
    """ Assert that get_short_name returns name"""
    user = User.objects.create_user('test_short_name@test.com', 'validpassword')
    user.name="Abc def"
    user.save()

    self.assertTrue(user.get_short_name() == user.name)

  def test_password_hashing(self):
    """ Assert password is not rehashed during saves """
    user = User.objects.create_user('user@email.com', 'validpassword')
    user.save()
    self.assertTrue(user.check_password('validpassword'))
    user.set_password('anotherpassword')
    self.assertTrue(user.check_password('anotherpassword'))
    user.save()
    self.assertTrue(user.check_password('anotherpassword'))

  def test_slug(self):
    """ Assert that during user creation slug matches the uuid """
    user = User.objects.create_user('user@email.com', 'validpassword')
    user.save()
    self.assertTrue(encode_uuid(user.uuid) == user.slug)



from ovp_users.serializers.user import UserCreateSerializer
from ovp_users.serializers.user import UserUpdateSerializer
from ovp_users.serializers.user import CurrentUserSerializer
from ovp_users.serializers.user import ShortUserPublicRetrieveSerializer
from ovp_users.serializers.user import LongUserPublicRetrieveSerializer
from ovp_users.serializers.user import UserProjectRetrieveSerializer
from ovp_users.serializers.user import UserApplyRetrieveSerializer

from ovp_users.serializers.password_recovery import RecoveryTokenSerializer
from ovp_users.serializers.password_recovery import RecoverPasswordSerializer

from ovp_users.serializers.profile import ProfileCreateUpdateSerializer
from ovp_users.serializers.profile import ProfileRetrieveSerializer



from django.core.exceptions import ValidationError
from django.contrib.auth.password_validation import validate_password
from django.contrib.auth import authenticate

from ovp_users import models
from ovp_users.helpers import get_settings, import_from_string
from ovp_users.models.profile import get_profile_model
from ovp_users.serializers.profile import get_profile_serializers
from ovp_users.serializers.profile import ProfileSearchSerializer
from ovp_uploads.serializers import UploadedImageSerializer
from ovp_projects.serializers.apply_user import ApplyUserRetrieveSerializer

from ovp_projects import models as model_project

from rest_framework import serializers
from rest_framework import permissions
from rest_framework import fields

class UserCreateSerializer(serializers.ModelSerializer):
  profile = get_profile_serializers()[0](required=False)
  slug = serializers.CharField(read_only=True)
  uuid = serializers.CharField(read_only=True)

  class Meta:
    model = models.User
    fields = ['uuid', 'name', 'email', 'password', 'phone', 'avatar', 'locale', 'profile', 'public', 'slug']
    extra_kwargs = {'password': {'write_only': True}}

  def validate(self, data):
    errors = dict()

    if data.get('password'):
      password = data.get('password', '')
      try:
        validate_password(password=password)
      except ValidationError as e:
        errors['password'] = list(e.messages)

    if errors:
      raise serializers.ValidationError(errors)

    return super(UserCreateSerializer, self).validate(data)

  def create(self, validated_data):
    profile_data = validated_data.pop('profile', {})

    # Create user
    user = models.User.objects.create(**validated_data)

    # Profile
    profile_data['user'] = user
    profile_sr = get_profile_serializers()[0](data=profile_data)
    profile = profile_sr.create(profile_data)

    return user

class UserUpdateSerializer(UserCreateSerializer):
  current_password = fields.CharField(write_only=True)
  profile = get_profile_serializers()[0](required=False)

  class Meta:
    model = models.User
    permission_classes = (permissions.IsAuthenticated,)
    fields = ['name', 'phone', 'password', 'avatar', 'current_password', 'locale', 'profile', 'public']
    extra_kwargs = {'password': {'write_only': True}}


  def validate(self, data):
    errors = dict()

    if data.get('password') or data.get('current_password'):
      current_password = data.pop('current_password', '')
      password = data.get('password', '')

      try:
        validate_password(password=password)
      except ValidationError as e:
        errors['password'] = list(e.messages)

      if not authenticate(email=self.context['request'].user.email, password=current_password):
        errors['current_password'] = ["Invalid password."]

    if errors:
      raise serializers.ValidationError(errors)

    return super(UserCreateSerializer, self).validate(data)

  def update(self, instance, data):
    ProfileModel = get_profile_model()
    profile_data = data.pop('profile', None)

    if profile_data:
      has_profile=False
      try:
        if instance.profile:
          has_profile=True
        else:
          has_profile=False
      except models.UserProfile.DoesNotExist:
        has_profile=False

      if has_profile:
        profile = instance.profile
      else:
        profile = ProfileModel(user=instance)
        profile.save()

      profile_sr = get_profile_serializers()[0](profile, data=profile_data)
      profile_sr.is_valid(raise_exception=True)
      profile = profile_sr.update(profile, profile_sr.validated_data)

    return super(UserUpdateSerializer, self).update(instance, data)


class CurrentUserSerializer(serializers.ModelSerializer):
  avatar = UploadedImageSerializer()
  profile = get_profile_serializers()[1]()

  class Meta:
    model = models.User
    fields = ['uuid', 'name', 'phone', 'avatar', 'email', 'locale', 'profile', 'slug']

class ShortUserPublicRetrieveSerializer(serializers.ModelSerializer):
  avatar = UploadedImageSerializer()

  class Meta:
    model = models.User
    fields = ['uuid', 'name', 'avatar', 'slug']

class LongUserPublicRetrieveSerializer(serializers.ModelSerializer):
  avatar = UploadedImageSerializer()
  profile = get_profile_serializers()[1]()
  applies = ApplyUserRetrieveSerializer(many=True, source="apply_set")

  class Meta:
    model = models.User
    fields = ['name', 'avatar', 'profile', 'slug', 'applies']

class UserProjectRetrieveSerializer(serializers.ModelSerializer):
  avatar = UploadedImageSerializer()

  class Meta:
    model = models.User
    fields = ['uuid', 'name', 'avatar', 'email', 'phone', 'slug']

class UserApplyRetrieveSerializer(serializers.ModelSerializer):
  avatar = UploadedImageSerializer()

  class Meta:
    model = models.User
    fields = ['uuid', 'name', 'avatar', 'phone', 'email']

class UserSearchSerializer(serializers.ModelSerializer):
  avatar = UploadedImageSerializer()
  profile = get_profile_serializers()[2]()

  class Meta:
    model = models.User
    fields = ['slug', 'name', 'avatar', 'profile']

def get_user_search_serializer():
  s = get_settings()
  class_path = s.get('USER_SEARCH_SERIALIZER', None)
  if class_path:
    return import_from_string(class_path)
  return UserSearchSerializer



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-09-07 21:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='Active'),
        ),
        migrations.AddField(
            model_name='user',
            name='is_email_verified',
            field=models.BooleanField(default=False, verbose_name='Email verified'),
        ),
        migrations.AddField(
            model_name='user',
            name='is_staff',
            field=models.BooleanField(default=False, verbose_name='Staff'),
        ),
        migrations.AddField(
            model_name='user',
            name='joined_date',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
        migrations.AddField(
            model_name='user',
            name='modified_date',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
    ]



from ovp_users import serializers
from ovp_users import models

from rest_framework import decorators
from rest_framework import mixins
from rest_framework import response
from rest_framework import viewsets
from rest_framework import permissions

import json

class UserResourceViewSet(mixins.CreateModelMixin, viewsets.GenericViewSet):
  """
  UserResourceViewSet resource endpoint
  """
  queryset = models.User.objects.all()
  lookup_field = 'slug'
  lookup_value_regex = '[^/]+' # default is [^/.]+ - here we're allowing dots in the url slug field

  def current_user_get(self, request, *args, **kwargs):
    queryset = self.get_object()
    serializer = self.get_serializer(queryset, context=self.get_serializer_context())
    return response.Response(serializer.data)

  def current_user_put(self, request, *args, **kwargs):
    partial = kwargs.pop('partial', False)
    instance = self.get_object()
    serializer = self.get_serializer(instance, data=request.data, partial=partial, context=self.get_serializer_context())
    serializer.is_valid(raise_exception=True)
    serializer.save()

    return response.Response(serializer.data)

  def current_user_patch(self, request, *args, **kwargs):
    kwargs['partial'] = True
    return self.current_user_put(request, *args, **kwargs)

  @decorators.list_route(url_path="current-user", methods=['GET', 'PATCH', 'PUT'])
  def current_user(self, request, *args, **kwargs):
    if request.method == 'GET':
      return self.current_user_get(request, *args, **kwargs)
    if request.method == 'PUT':
      return self.current_user_put(request, *args, **kwargs)
    if request.method == 'PATCH':
      return self.current_user_patch(request, *args, **kwargs)

  def get_object(self):
    request = self.get_serializer_context()['request']
    if self.action == 'current_user':
      return self.get_queryset().get(pk=request.user.pk)

    # Shouldn't really be called for current implementation
    # but here as fail-safe for future updates
    return super(UserResourceViewSet, self).get_object() #pragma: no cover

  # We need to override get_permissions and get_serializer_class to work
  # with multiple serializers and permissions
  def get_permissions(self):
    request = self.get_serializer_context()['request']
    if self.action == 'create':
      self.permission_classes = []
    elif self.action in ['current_user']:
      self.permission_classes = [permissions.IsAuthenticated, ]

    return super(UserResourceViewSet, self).get_permissions()

  def get_serializer_class(self):
    request = self.get_serializer_context()['request']

    if self.action == 'create':
      return serializers.UserCreateSerializer

    if self.action == 'current_user':
      if request.method == "GET":
        return serializers.CurrentUserSerializer
      elif request.method in ["PUT", "PATCH"]:
        return serializers.UserUpdateSerializer


class PublicUserResourceViewSet(mixins.RetrieveModelMixin, viewsets.GenericViewSet):
  """
  PublicUserResourceViewSet resource endpoint
  """
  queryset = models.User.objects.filter(public=True)
  serializer_class = serializers.LongUserPublicRetrieveSerializer
  lookup_field = 'slug'
  lookup_value_regex = '[^/]+' # default is [^/.]+ - here we're allowing dots in the url slug field



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-16 17:06
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_core', '0007_lead'),
        ('ovp_users', '0014_user_locale'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('full_name', models.CharField(blank=True, max_length=300, null=True, verbose_name='Full name')),
                ('about', models.TextField(blank=True, null=True, verbose_name='About me')),
                ('skills', models.ManyToManyField(to='ovp_core.Skill')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-21 13:47
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0017_userprofile_causes'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-21 13:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0018_user_uuid'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='public',
            field=models.BooleanField(default=True, verbose_name='Public Profile'),
        ),
    ]



import re

from django.test import TestCase
from django.core import mail

from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from ovp_users.tests.helpers import authenticate
from ovp_users.tests.helpers import create_user
from ovp_users.tests.helpers import create_token


class RecoveryTokenViewSetTestCase(TestCase):
  def test_can_create_token(self):
    """Assert that it's possible to create an recovery token"""
    user = create_user('test@recovery.token')
    response = create_token()
    self.assertTrue(response.data['success'] == True)

  def test_cant_create_6_tokens(self):
    """Assert that it's impossible to create more than 5 tokens in less than 60 minutes"""
    user = create_user('test2@recovery.token')
    response = None
    for i in range(6):
      response = create_token('test2@recovery.token')

    self.assertFalse(response.data.get('success', False) == True)

  def test_token_for_invalid_user(self):
    """Assert the server hides the fact user don't exist when requesting token"""
    response = create_token('invaliduser@invalid.com')
    self.assertTrue(response.data['success'] == True)
    self.assertTrue(len(mail.outbox) == 0)

class RecoverPasswordViewSetTestCase(TestCase):
  def test_can_recover_password(self):
    """Assert the user can recover his password with a valid token"""
    # Request token
    user = create_user('test_can_recover@password.com')

    mail.outbox = [] # Clear outbox
    response = create_token('test_can_recover@password.com')

    # Get Token from mailbox
    email_content = mail.outbox[0].alternatives[0][0]
    token = re.search('[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}', email_content).group(0)

    # Recover Password
    data = {
      'email': 'test_can_recover@password.com',
      'token': token,
      'new_password': 'newpwvalidpw*'
    }

    client = APIClient()
    response = client.post(reverse('recover-password-list'), data, format="json")
    self.assertTrue(response.data['message'] == 'Password updated.')

    # Test authentication new password
    auth = authenticate('test_can_recover@password.com', 'newpwvalidpw*')
    self.assertTrue(auth.data['token'] != None)

  def test_cant_recover_with_empty_password(self):
    """Assert that it's impossible to update password through recovery to an empty password"""
    # Request token
    user = create_user('test_cant_recover_empty@password.com')

    mail.outbox = [] # Clear outbox
    response = create_token('test_cant_recover_empty@password.com')

    # Get Token from mailbox
    email_content = mail.outbox[0].alternatives[0][0]
    token = re.search('[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}', email_content).group(0)

    # Recover Password
    data = {
      'email': 'test_cant_recover_empty@password.com',
      'token': token,
      'new_password': ''
    }

    client = APIClient()
    response = client.post(reverse('recover-password-list'), data, format="json")
    self.assertTrue(response.data['message'] == 'Empty password not allowed.')
    self.assertTrue(response.status_code == 400)

  def test_cant_recover_invalid_token(self):
    """Assert that it's impossible to update password through recovery with an invalid user token"""
    # Request token
    user = create_user('test_cant_recover_invalid@password.com')
    response = create_token('test_cant_recover_invalid@password.com')

    # Recover Password
    data = {
      'email': 'test_cant_recover_invalid@password.com',
      'token': 'invalid_token',
      'new_password': 'newpassword'
    }

    client = APIClient()
    response = client.post(reverse('recover-password-list'), data, format="json")
    self.assertTrue(response.data['message'] == 'Invalid email or token.')
    self.assertTrue(response.status_code == 401)

  def test_cant_recover_invalid_password(self):
    """Assert that it's impossible to update password through recovery with an invalid user token"""
    # Request token
    user = create_user('test_cant_recover_invalid_pw@password.com')

    mail.outbox = [] # Clear outbox
    response = create_token('test_cant_recover_invalid_pw@password.com')

    # Get Token from mailbox
    email_content = mail.outbox[0].alternatives[0][0]
    token = re.search('[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}', email_content).group(0)

    # Recover Password
    data = {
      'email': 'test_cant_recover_invalid_pw@password.com',
      'token': token,
      'new_password': 'ab'
    }

    client = APIClient()
    response = client.post(reverse('recover-password-list'), data, format="json")
    self.assertTrue(response.data['message'] == 'Invalid password.')
    self.assertTrue(response.status_code == 400)



from ovp_users.views.user import UserResourceViewSet
from ovp_users.views.user import PublicUserResourceViewSet
from ovp_users.views.password_recovery import RecoveryTokenViewSet
from ovp_users.views.password_recovery import RecoverPasswordViewSet
from ovp_users.views.messages import UserMessageViewSet



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-04-05 14:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0023_auto_20170224_1829'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='public',
            field=models.BooleanField(default=True, verbose_name='Public'),
        ),
    ]



from django.test import TestCase

from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from ovp_users import models
from ovp_users.tests.helpers import authenticate
from ovp_users.tests.helpers import create_user


class UserResourceViewSetTestCase(TestCase):
  def test_can_create_user(self):
    """Assert that it's possible to create an user"""
    response = create_user()
    self.assertTrue(response.data['uuid'])

  def test_cant_create_user_duplicated_email(self):
    """Assert that it's not possible to create an user with a repeated email"""
    response = create_user()
    self.assertTrue(response.data['uuid'])

  def test_cant_create_user_invalid_password(self):
    """Assert that it's not possible to create an user with a repeated email"""
    response = create_user('test_cant_create_invalid_password@test.com', 'abc')
    self.assertTrue(len(response.data['password']) > 0)
    self.assertTrue(isinstance(response.data['password'], list))

  def test_doesnt_return_password_on_user_creation(self):
    """Assert that the serializer does not return user hashed password """
    response = create_user()
    self.assertTrue(response.data.get('password', None) == None)

  def test_cant_patch_password_without_current_password(self):
    """Assert that it's not possible to update user password without the current password"""
    response = create_user('test_can_patch_password@test.com', 'abcabcabc')
    u = models.User.objects.get(uuid=response.data['uuid'])

    data = {'password': 'pwpw12341234'}
    client = APIClient()
    client.force_authenticate(user=u)
    response = client.patch(reverse('user-current-user'), data, format="json")
    self.assertTrue(response.status_code == 400)
    self.assertTrue(response.data["current_password"] == ["Invalid password."])

  def test_can_patch_password(self):
    """Assert that it's possible to update user password"""
    response = create_user('test_can_patch_password@test.com', 'abcabcabc')
    u = models.User.objects.get(uuid=response.data['uuid'])

    data = {'password': 'pwpw12341234', 'current_password': 'abcabcabc'}
    client = APIClient()
    client.force_authenticate(user=u)
    response = client.patch(reverse('user-current-user'), data, format="json")
    self.assertTrue(response.status_code == 200)

    response = authenticate('test_can_patch_password@test.com', data['password'])
    self.assertTrue(response.data['token'] != None)

  def test_can_put_password(self):
    """Assert that it's possible to update user password"""
    response = create_user('test_can_put_password@test.com', 'abcabcabc')
    u = models.User.objects.get(uuid=response.data['uuid'])

    data = {'name': 'abc', 'password': 'pwpw12341234', 'current_password': 'abcabcabc'}
    client = APIClient()
    client.force_authenticate(user=u)
    response = client.put(reverse('user-current-user'), data, format="json")
    self.assertTrue(response.status_code == 200)

    response = authenticate('test_can_put_password@test.com', data['password'])
    self.assertTrue(response.data['token'] != None)

  def test_cant_update_invalid_password(self):
    """Assert that it's impossible to update user password to a invalid password"""
    response = create_user('test_can_put_password@test.com', 'abcabcabc')
    u = models.User.objects.get(uuid=response.data['uuid'])

    data = {'name': 'abc', 'password': 'abc', 'current_password': 'abcabcabc'}
    client = APIClient()
    client.force_authenticate(user=u)
    response = client.put(reverse('user-current-user'), data, format="json")
    self.assertTrue(response.status_code == 400)
    self.assertTrue(len(response.data['password']) > 0)
    self.assertTrue(isinstance(response.data['password'], list))

  def test_can_get_current_user(self):
    """Assert that authenticated users can get associated info"""
    user = create_user('test_can_get_current_user@test.com', 'validpassword')

    client = APIClient()
    client.login(username='test_can_get_current_user@test.com', password='validpassword')
    response = client.get(reverse('user-current-user'), {}, format="json")
    self.assertTrue(response.data.get('email', None))
    self.assertTrue(response.data.get('name', None))

  def test_can_create_hidden_user(self):
    """Assert that it's possible to create a hidden user"""
    response = create_user(extra_data={'public': False})
    self.assertTrue(response.data['public'] == False)

  def test_can_set_user_to_hidden(self):
    """Assert that it's possible to set public user to hidden user"""
    response = create_user()
    self.assertTrue(response.data['public'] == True)

    user = models.User.objects.get(uuid=response.data['uuid'])
    client = APIClient()
    client.force_authenticate(user=user)
    response = client.patch(reverse('user-current-user'), {'public': False}, format="json")
    self.assertTrue(response.data['public'] == False)

  def test_can_retrieve_public_user(self):
    """ Assert it's possible to retrieve a public profile """
    response = create_user()

    client = APIClient()
    response = client.get(reverse('public-users-detail', [response.data['slug']]), format="json")
    self.assertTrue(response.data['slug'])
    self.assertTrue("applies" in response.data)

  def test_cant_retrieve_hidden_user(self):
    """ Assert it's not possible to retrieve a hidden profile """
    response = create_user(extra_data={'public': False})

    client = APIClient()
    response = client.get(reverse('public-users-detail', [response.data['slug']]), format="json")
    self.assertTrue(response.status_code == 404)

  def test_ask_for_credentials(self):
    """Assert that unauthenticated users can't get current user info"""
    client = APIClient()
    response = client.get(reverse('user-current-user'), {}, format="json")
    self.assertTrue(response.data['detail'] == 'Authentication credentials were not provided.')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-04-05 14:50
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0024_user_public'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='userprofile',
            name='public',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-08 21:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0012_merge_20170112_2144'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='email',
            field=models.EmailField(max_length=190, unique=True, verbose_name='Email'),
        ),
    ]



from django.db import models
from django.utils.translation import ugettext_lazy as _

from ovp_users.helpers import get_settings, import_from_string

gender_choices = (
  ("male", "Male"),
  ("female", "Female"),
)

class UserProfile(models.Model):
  user = models.OneToOneField("User", blank=True, null=True, related_name="%(app_label)s_%(class)s_profile")
  full_name = models.CharField(_("Full name"), max_length=300, null=True, blank=True)
  skills = models.ManyToManyField("ovp_core.Skill")
  causes = models.ManyToManyField("ovp_core.Cause")
  about = models.TextField(_("About me"), null=True, blank=True)
  gender = models.CharField(_("Gender"), max_length=10, choices=gender_choices, default=None, null=True, blank=True)

def get_profile_model():
  s = get_settings()
  class_path = s.get("PROFILE_MODEL", None)
  if class_path:
    return import_from_string(class_path)
  return UserProfile



from ovp_users.tests.test_views.user import UserResourceViewSetTestCase
from ovp_users.tests.test_views.auth import JWTAuthTestCase
from ovp_users.tests.test_views.profile import ProfileTestCase
from ovp_users.tests.test_views.password_recovery import RecoveryTokenViewSetTestCase
from ovp_users.tests.test_views.password_recovery import RecoverPasswordViewSetTestCase



from rest_framework.reverse import reverse
from rest_framework.test import APIClient

def create_user(email="validemail@gmail.com", password="validpassword", extra_data={}):
  data = {
    'name': 'Valid Name',
    'email': email,
    'password': password
  }
  data = dict(data, **extra_data)

  client = APIClient()
  return client.post(reverse('user-list'), data, format="json")


def create_user_with_profile(email="validemail@gmail.com", password="validpassword", profile={}):
  data = {
    'name': 'Valid Name',
    'email': email,
    'password': password,
    'profile': profile
  }

  client = APIClient()
  return client.post(reverse('user-list'), data, format="json")

def authenticate(email='test_can_login@test.com', password='validpassword'):
  data = {
    'email': email,
    'password': password
  }

  client = APIClient()
  return client.post('/api-token-auth/', data, format="json")


def create_token(email='test@recovery.token'):
  data = {
    'email': email,
  }

  client = APIClient()
  return client.post(reverse('recovery-token-list'), data, format="json")



from ovp_core.emails import BaseMail

class UserMail(BaseMail):
  """
  This class is responsible for firing emails for Users
  """
  def __init__(self, user, async_mail=None):
    super(UserMail, self).__init__(user.email, async_mail, user.locale)

  def sendWelcome(self, context={}):
    """
    Sent when user registers
    """
    return self.sendEmail('welcome', 'Welcome', context)


  def sendRecoveryToken(self, context):
    """
    Sent when volunteer requests recovery token
    """
    return self.sendEmail('recoveryToken', 'Password recovery', context)


  def sendMessageToAnotherVolunteer(self, context):
    """
    Sent when volunteer make contact with another volunteer
    """
    return self.sendEmail('messageToVolunteer', 'Volunteer Message', context)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-01-12 21:44
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0011_auto_20170111_1633'),
        ('ovp_users', '0011_auto_20170112_1417'),
    ]

    operations = [
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-01-12 14:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0010_user_avatar'),
    ]

    operations = [
        migrations.AlterField(
            model_name='passwordrecoverytoken',
            name='created_date',
            field=models.DateTimeField(auto_now_add=True, null=True, verbose_name='Created date'),
        ),
        migrations.AlterField(
            model_name='passwordrecoverytoken',
            name='used_date',
            field=models.DateTimeField(blank=True, default=None, null=True, verbose_name='Used date'),
        ),
        migrations.AlterField(
            model_name='user',
            name='avatar',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='avatar_user', to='ovp_uploads.UploadedImage', verbose_name='avatar'),
        ),
        migrations.AlterField(
            model_name='user',
            name='joined_date',
            field=models.DateTimeField(auto_now_add=True, null=True, verbose_name='Joined date'),
        ),
        migrations.AlterField(
            model_name='user',
            name='modified_date',
            field=models.DateTimeField(auto_now=True, null=True, verbose_name='Modified date'),
        ),
    ]



from django.test import TestCase

from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from ovp_users.models import User
from ovp_users.tests.helpers import create_user
from ovp_users.tests.helpers import create_user_with_profile

class ProfileTestCase(TestCase):
  def setUp(self):
    self.profile = {
      'full_name': 'Test User',
      'skills': [{'id': 1}, {'id': 2, 'name': 'test'}],
      'causes': [{'id': 1}, {'id': 2, 'name': 'test'}],
      'about': 'I\'m a test user and that\'s all you must know about me',
      'gender': 'male'
    }
    self.client = APIClient()


  def test_can_create_user_with_profile(self):
    """ Assert profile data gets saved when creating user """
    response = create_user_with_profile(profile=self.profile)
    self.assertTrue(response.data['profile']['full_name'] == self.profile['full_name'])
    self.assertTrue(response.data['profile']['about'] == self.profile['about'])
    self.assertTrue(response.data['profile']['gender'] == 'male')
    self._assert_causes_and_skills_in_response(response)
    return response


  def test_current_user_route_returns_profile(self):
    """ Assert profile data gets returned when fetching current user """
    user = create_user_with_profile(profile=self.profile)
    self.client.force_authenticate(User.objects.get(uuid=user.data['uuid']))

    response = self.client.get(reverse('user-current-user'), {}, format="json")
    self.assertTrue(response.data['profile']['full_name'] == self.profile['full_name'])
    self.assertTrue(response.data['profile']['about'] == self.profile['about'])
    self.assertTrue(response.data['profile']['gender'] == 'male')
    self._assert_causes_and_skills_in_response(response)


  def test_can_update_profile(self):
    """ Assert it's possible to update profile data """
    user = self.test_can_create_user_with_profile()
    self.client.force_authenticate(User.objects.get(uuid=user.data['uuid']))
    self._test_can_update()


  def test_can_update_inexistent_profile(self):
    """ Assert it's possible to update profile data even if profile is inexistent """
    user = User.objects.create(email='test_user@email.com', password='test_password')
    self.client.force_authenticate(user)
    self._test_can_update()


  def test_skills_validation_on_create(self):
    """ Assert it's impossible to associate with invalid skills on profile creation """
    profile = self.profile
    profile['skills'] = [{'id': 999}, {'id': 998}]
    response = create_user_with_profile(profile=profile)
    self.assertTrue(response.status_code == 400)
    self.assertTrue(response.data['profile']['skills'][0]['id'] == ["Skill with 'id' 999 does not exist."])
    self.assertTrue(response.data['profile']['skills'][1]['id'] == ["Skill with 'id' 998 does not exist."])


  def test_causes_validation_on_create(self):
    """ Assert it's impossible to associate with invalid causes on profile creation"""
    profile = self.profile
    profile['causes'] = [{'id': 999}, {'id': 998}]
    response = create_user_with_profile(profile=profile)
    self.assertTrue(response.status_code == 400)
    self.assertTrue(response.data['profile']['causes'][0]['id'] == ["Cause with 'id' 999 does not exist."])
    self.assertTrue(response.data['profile']['causes'][1]['id'] == ["Cause with 'id' 998 does not exist."])


  def test_skills_validation_on_update(self):
    """ Assert it's impossible to associate with invalid skills on profile update """
    user = self.test_can_create_user_with_profile()
    self.client.force_authenticate(User.objects.get(uuid=user.data['uuid']))

    data = {
      'profile': {
        'skills': [{'id': 999}, {'id': 998}]
      }
    }

    response = self.client.patch(reverse('user-current-user'), data, format="json")
    self.assertTrue(response.status_code == 400)
    self.assertTrue(response.data['profile']['skills'][0]['id'] == ["Skill with 'id' 999 does not exist."])
    self.assertTrue(response.data['profile']['skills'][1]['id'] == ["Skill with 'id' 998 does not exist."])


  def test_causes_validation_on_update(self):
    """ Assert it's impossible to associate with invalid causes on profile update """
    user = self.test_can_create_user_with_profile()
    self.client.force_authenticate(User.objects.get(uuid=user.data['uuid']))

    data = {
      'profile': {
        'causes': [{'id': 999}, {'id': 998}]
      }
    }

    response = self.client.patch(reverse('user-current-user'), data, format="json")
    self.assertTrue(response.status_code == 400)
    self.assertTrue(response.data['profile']['causes'][0]['id'] == ["Cause with 'id' 999 does not exist."])
    self.assertTrue(response.data['profile']['causes'][1]['id'] == ["Cause with 'id' 998 does not exist."])

  def _assert_causes_and_skills_in_response(self, response):
    self.assertTrue(response.data['profile']['skills'][0]['id'] == 1)
    self.assertTrue('name' in response.data['profile']['skills'][0])
    self.assertTrue(response.data['profile']['skills'][1]['id'] == 2)
    self.assertTrue('name' in response.data['profile']['skills'][1])

    self.assertTrue(response.data['profile']['causes'][0]['id'] == 1)
    self.assertTrue('name' in response.data['profile']['causes'][0])
    self.assertTrue(response.data['profile']['causes'][1]['id'] == 2)
    self.assertTrue('name' in response.data['profile']['causes'][1])


  def _test_can_update(self):
    data = {
      'profile': {
        'full_name': 'New name',
        'about': 'New about',
        'causes': [{'id': 3}, {'id': 4}],
        'skills': [{'id': 3}, {'id': 4}],
        'gender': 'female'
      }
    }

    response = self.client.patch(reverse('user-current-user'), data, format="json")
    self.assertTrue(response.data['profile']['full_name'] == data['profile']['full_name'])
    self.assertTrue(response.data['profile']['about'] == data['profile']['about'])
    self.assertTrue(response.data['profile']['gender'] == 'female')

    self.assertTrue(response.data['profile']['skills'][0]['id'] == 3)
    self.assertTrue(response.data['profile']['skills'][1]['id'] == 4)
    self.assertTrue(response.data['profile']['causes'][0]['id'] == 3)
    self.assertTrue(response.data['profile']['causes'][1]['id'] == 4)



from ovp_users import serializers
from ovp_users import emails

from rest_framework import decorators
from rest_framework import mixins
from rest_framework import response
from rest_framework import viewsets
from rest_framework import permissions
from rest_framework import pagination

class UserMessageViewSet(mixins.CreateModelMixin, viewsets.GenericViewSet):
  email = ''
  locale = ''

  def mailing(self, async_mail=None):
    return emails.UserMail(self, async_mail)

  def create(self, request, *args, **kwargs):
    self.email = request.data['emailTo']
    self.mailing().sendMessageToAnotherVolunteer(request.data)
    return response.Response(True)


# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='ovp-users',
    version='1.1.7',
    author=u'Atados',
    author_email='arroyo@atados.com.br',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/OpenVolunteeringPlatform/ovp-users',
    download_url='https://github.com/OpenVolunteeringPlatform/ovp-users/tarball/1.1.7',
    license='AGPL',
    description='This module has core functionality for' + \
                ' ovp users, such as authentication, user management' + \
                ' password recovery and registration',
    long_description=open('README.rst', encoding='utf-8').read(),
    zip_safe=False,
    install_requires = [
      'Django>=1.10.1,<1.11.0',
      'djangorestframework>=3.5.0,<3.6.0',
      'djangorestframework-jwt>=1.8.0,<2.0.0',
      'python-dateutil>=2.5.3,<2.7.0',
      'codecov>=2.0.5,<2.1.0',
      'coverage>=4.2,<4.4.0',
      'ovp-core>=1.2.4,<2.0.0',
      'ovp-uploads>=1.0.0,<2.0.0',
      'ovp-projects>=1.2.8,<2.0.0',
      'ovp-organizations>=1.0.11,<2.0.0',
      'shortuuid>=0.5.0,<1.0.0',
    ]
)



from django.conf.urls import url, include
from rest_framework import routers
from rest_framework_jwt.views import obtain_jwt_token
from ovp_users import views

router = routers.DefaultRouter()
router.register(r'users', views.UserResourceViewSet, 'user')
router.register(r'users/recovery-token', views.RecoveryTokenViewSet, 'recovery-token')
router.register(r'users/recover-password', views.RecoverPasswordViewSet, 'recover-password')
router.register(r'users/send-message', views.UserMessageViewSet, 'send-message')
router.register(r'public-users', views.PublicUserResourceViewSet, 'public-users')

urlpatterns = [
  url(r'^', include(router.urls)),
  url(r'^api-token-auth/', obtain_jwt_token),
]



from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _

from ovp_users.models import User


class UserAdmin(admin.ModelAdmin):
  fields = [
    ('id', 'name', 'email'), 'slug', 'phone',
    ('is_staff','is_superuser','is_active','is_email_verified',)
    ]

  list_display = [
    'id', 'email', 'name', 'last_login', 'is_active', 'is_staff', 'is_email_verified'
    ]

  list_filter = [
    'is_active', 'is_staff', 'last_login', 'joined_date'
    ]

  list_editable = [
    'is_active', 'is_staff', 'is_email_verified'
    ]

  search_fields = [
    'email', 'name'
    ]

  readonly_fields = ['id']
  raw_id_fields = []


admin.site.register(User, UserAdmin)





# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-21 19:23
from __future__ import unicode_literals

from django.db import migrations
import uuid

def gen_uuid(apps, schema_editor):
    Model = apps.get_model('ovp_users', 'User')
    for user in Model.objects.all():
        user.uuid = uuid.uuid4()
        user.save()

class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0019_userprofile_public'),
    ]

    operations = [
        migrations.RunPython(gen_uuid, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-20 22:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_core', '0007_lead'),
        ('ovp_users', '0016_auto_20170216_1930'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='causes',
            field=models.ManyToManyField(to='ovp_core.Cause'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-04-18 16:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0025_remove_userprofile_public'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='genre',
            field=models.CharField(blank=True, choices=[('male', 'Male'), ('female', 'Female')], default=None, max_length=10, null=True, verbose_name='Genre'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-15 18:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0013_auto_20170208_2118'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='locale',
            field=models.CharField(blank=True, default='en', max_length=8, verbose_name='Locale'),
        ),
    ]



from django.test import TestCase

from ovp_users.tests.helpers import authenticate
from ovp_users.tests.helpers import create_user
from ovp_users.tests.helpers import create_token


class JWTAuthTestCase(TestCase):
  def test_can_login(self):
    """Assert that it's possible to login"""
    user = create_user('test_can_login@test.com', 'validpassword')
    response = authenticate()
    self.assertTrue(response.data['token'] != None)

  def test_cant_login_wrong_password(self):
    """Assert that it's not possible to login with wrong password"""
    user = create_user('test_can_login@test.com', 'invalidpassword')
    response = authenticate()
    self.assertTrue(response.data['non_field_errors'][0] == 'Unable to login with provided credentials.')
